==> eventbus-0.0.21.tgz: package/lib/Store.js <==
"use strict";

const EventEmitter = require('events');
const Promise      = require('bluebird');
const debug        = require('debug')('eventbus:store');
const pg           = require('pg');
const _            = require('lodash');
const Cursor       = require('pg-cursor');
const CFError      = require('cf-errors');
const tables       = require('./tables');

const INIT_RETRY_INTERVAL = 5; // in seconds
const SCHEMA              = 'eventstore.events';

class Store extends EventEmitter {

    constructor(microServiceName, options) {
        super();
        this.microServiceName = microServiceName;
        this.host             = options.host;
        this.database         = options.database;
        this.user             = options.user;
        this.password         = options.password;
        this.readyPromise     = new Promise((resolve, reject) => {
            this.readyDeferred = {
                resolve: () => {
                    resolve();
                },
                reject: () => {
                    reject();
                }
            };
        });
    }

    async quit() {
        debug(`Pool client state before closing. total: ${this.pool.totalCount}. idle: ${this.pool.idleCount}. active: ${this.pool.activeCount}`);
        await this.pool.end();
        debug(`Pool client state after closing. total: ${this.pool.totalCount}. idle: ${this.pool.idleCount}. active: ${this.pool.activeCount}`);
        debug('store ');
    }

    init() {
        const config = {
            user: this.user,
            database: this.database,
            password: this.password,
            host: this.host,
            port: 5432,
            max: 10,
            idleTimeoutMillis: 30000, // how long a client is allowed to remain idle before being closed
        };
        this.pool    = new pg.Pool(config);

        this.pool.on('error', (err) => {
            this.emit('error', new CFError({
                cause: err,
                message: 'postgres pool error'
            }));
        });

        this._assertSystemTables()
            .then(() => {
                debug(`ready`);
                this.emit('ready');
            })
            .catch((err) => {
                this.emit('error', new CFError({
                    cause: err,
                    message: `Initialization failed`
                }));
                debug(`initialization failed trying again in ${INIT_RETRY_INTERVAL} seconds`);
                setTimeout(() => {
                    this.init();
                }, INIT_RETRY_INTERVAL * 1000);
            });
    }

    async _assertSystemTables() {
        await this.pool.query(tables);
        debug('system tables asserted');
    }

    /**
     * extends the pg cursor with an easy api for the client
     * @param cursor
     * @param queryCondition
     * @private
     */
    _extendPgCursor(cursor, client, queryCondition) {
        cursor.read = Promise.promisify(cursor.read.bind(cursor));

        cursor.count = async () => {
            const countResults = await this.pool.query(`SELECT COUNT(*) FROM ${SCHEMA} ${queryCondition}`);
            const totalRows    = Number(countResults.rows[0].count);
            return totalRows;
        };

        cursor.next = async () => {
            const rowResult = await cursor.read(1);
            const row       = rowResult[0];
            return row;
        };

        cursor.close = async () => {
            await client.release();
        }
    }

    async save(eventName, event) {
        let data = [
            { field: 'event', value: eventName },
            { field: 'props', value: _.get(event, 'props', {}) },
            { field: 'aggregate_id', value: _.get(event, 'aggregateId', '').toString() },
            { field: 'timestamp', value: new Date() },
            { field: 'publisher', value: this.microServiceName }
        ];

        if (event.accountId) {
            data.push({ field: 'account_id', value: _.get(event, 'accountId', '').toString() });
        }

        if (event.userId) {
            data.push({ field: 'user_id', value: _.get(event, 'userId', '').toString() });
        }

        let fieldString  = '';
        let valuesString = '';
        let dataArr      = [];
        data.forEach((d, index) => {
            if (!index) {
                fieldString += `${d.field}`;
                valuesString += `$${index + 1}`;
            } else {
                fieldString += `, ${d.field}`;
                valuesString += `, $${index + 1}`;
            }
            dataArr.push(d.value);
        });

        const storeQuery = `INSERT INTO ${SCHEMA} (${fieldString}) VALUES (${valuesString})`;
        await this.pool.query(storeQuery, dataArr);
    }

    async getAllAggregateEvents(aggregateId, fromIndex) {
        try {
            const queryCondition = `WHERE id >= ${fromIndex} AND aggregate_id = '${aggregateId}'`;

            const client = await this.pool.connect();

            const cursor = await client.query(new Cursor(`SELECT * FROM ${SCHEMA} ${queryCondition} ORDER BY timestamp`));

            this._extendPgCursor(cursor, client, queryCondition);

            return cursor;
        }
        catch (err) {
            throw new CFError({
                cause: err,
                message: `Failed to perform getAllAggregateEvents with aggregateId: ${aggregateId}`
            });
        }
    }

    async getAllEventsByName(name, fromIndex) {
        try {
            const queryCondition = `WHERE id >= ${fromIndex} AND event LIKE '${name}%'`;

            const client = await this.pool.connect();

            const cursor = await client.query(new Cursor(`SELECT * FROM ${SCHEMA} ${queryCondition} ORDER BY timestamp`));

            this._extendPgCursor(cursor, client, queryCondition);

            await cursor.count();

            return cursor;
        }
        catch (err) {
            throw new CFError({
                cause: err,
                message: `Failed to perform getAllEventsByName with name: ${name}`
            });
        }
    }
}




module.exports = Store;

==> eventbus-0.0.26.tgz: package/lib/Store.js <==
"use strict";

const EventEmitter = require('events');
const Promise      = require('bluebird');
const debug        = require('debug')('eventbus:store');
const pg           = require('pg');
const _            = require('lodash');
const Cursor       = require('pg-cursor');
const CFError      = require('cf-errors');
const tables       = require('./tables');

const INIT_RETRY_INTERVAL = 5; // in seconds
const SCHEMA              = 'eventstore.events';

class Store extends EventEmitter {

    constructor(microServiceName, options) {
        super();
        this.microServiceName = microServiceName;
        this.host             = options.host;
        this.database         = options.database;
        this.user             = options.user;
        this.password         = options.password;
        this.readyPromise     = new Promise((resolve, reject) => {
            this.readyDeferred = {
                resolve: () => {
                    resolve();
                },
                reject: () => {
                    reject();
                }
            };
        });
    }

    async quit() {
        debug(`Pool client state before closing. total: ${this.pool.totalCount}. idle: ${this.pool.idleCount}. active: ${this.pool.activeCount}`);
        await this.pool.end();
        debug(`Pool client state after closing. total: ${this.pool.totalCount}. idle: ${this.pool.idleCount}. active: ${this.pool.activeCount}`);
        debug('store ');
    }

    init() {
        const config = {
            user: this.user,
            database: this.database,
            password: this.password,
            host: this.host,
            port: 5432,
            max: 10,
            idleTimeoutMillis: 30000, // how long a client is allowed to remain idle before being closed
        };
        this.pool    = new pg.Pool(config);

        this.pool.on('error', (err) => {
            this.emit('error', new CFError({
                cause: err,
                message: 'postgres pool error'
            }));
        });

        // this._assertSystemTables()
        //     .then(() => {
        //         debug(`ready`);
        //         this.emit('ready');
        //     })
        //     .catch((err) => {
        //         this.emit('error', new CFError({
        //             cause: err,
        //             message: `Initialization failed`
        //         }));
        //         debug(`initialization failed trying again in ${INIT_RETRY_INTERVAL} seconds`);
        //         setTimeout(() => {
        //             this.init();
        //         }, INIT_RETRY_INTERVAL * 1000);
        //     });
    }

    async _assertSystemTables() {
        await this.pool.query(tables);
        debug('system tables asserted');
    }

    /**
     * extends the pg cursor with an easy api for the client
     * @param cursor
     * @param queryCondition
     * @private
     */
    _extendPgCursor(cursor, client, queryCondition) {
        cursor.read = Promise.promisify(cursor.read.bind(cursor));

        cursor.count = async () => {
            const countResults = await this.pool.query(`SELECT COUNT(*) FROM ${SCHEMA} ${queryCondition}`);
            const totalRows    = Number(countResults.rows[0].count);
            return totalRows;
        };

        cursor.next = async () => {
            const rowResult = await cursor.read(1);
            const row       = rowResult[0];
            return row;
        };

        cursor.close = async () => {
            await client.release();
        }
    }

    async save(eventName, event) {
        let data =  [
            { field: 'event', value: eventName },
            { field: 'props', value: _.get(event, 'props', {}) },
            { field: 'aggregate_id', value: _.get(event, 'aggregateId', '').toString() },
            { field: 'timestamp', value: _.get(event, 'timestamp', new Date()) },
            { field: 'publisher', value: _.get(event, 'publisher', this.microServiceName) }
        ];

        if (event.accountId) {
            data.push({ field: 'account_id', value: _.get(event, 'accountId', '').toString() });
        }

        if (event.userId) {
            data.push({ field: 'user_id', value: _.get(event, 'userId', '').toString() });
        }

        let fieldString  = '';
        let valuesString = '';
        let dataArr      = [];
        data.forEach((d, index) => {
            if (!index) {
                fieldString += `${d.field}`;
                valuesString += `$${index + 1}`;
            } else {
                fieldString += `, ${d.field}`;
                valuesString += `, $${index + 1}`;
            }
            dataArr.push(d.value);
        });

        const storeQuery = `INSERT INTO ${SCHEMA} (${fieldString}) VALUES (${valuesString})`;
        await this.pool.query(storeQuery, dataArr);
    }

    async getAllAggregateEvents(aggregateId, fromIndex) {
        try {
            const queryCondition = `WHERE id >= ${fromIndex} AND aggregate_id = '${aggregateId}'`;

            const client = await this.pool.connect();

            const cursor = await client.query(new Cursor(`SELECT * FROM ${SCHEMA} ${queryCondition} ORDER BY timestamp`));

            this._extendPgCursor(cursor, client, queryCondition);

            return cursor;
        }
        catch (err) {
            throw new CFError({
                cause: err,
                message: `Failed to perform getAllAggregateEvents with aggregateId: ${aggregateId}`
            });
        }
    }

    async getAllEventsByName(name, fromIndex) {
        try {
            const queryCondition = `WHERE id >= ${fromIndex} AND event LIKE '${name}%'`;

            const client = await this.pool.connect();

            const cursor = await client.query(new Cursor(`SELECT * FROM ${SCHEMA} ${queryCondition} ORDER BY timestamp`));

            this._extendPgCursor(cursor, client, queryCondition);

            await cursor.count();

            return cursor;
        }
        catch (err) {
            throw new CFError({
                cause: err,
                message: `Failed to perform getAllEventsByName with name: ${name}`
            });
        }
    }
}




module.exports = Store;
